module OrientDb
  class LearningObjectRepository < Base
    include OrientDb::Methods::EdgeMethods

    def increment_views(user, learning_object)
      create_edge "Views", user.rid, learning_object.id
    end

    #
    # Usage:
    #   repository.for(:learning_objects).like @learning_object, current_user
    #
    def like(user, learning_object)
      create_edge "Likes", user.rid, learning_object.id
    end

    # Example:
    #   list = repository.for(:learning_objects).all
    #   list.each do |learning_object|
    #     learning_object.inspect <LearningObject model>
    #   end
    def all
      learning_objects_hash = connection.query "SELECT FROM LearningObject"
      build_learning_objects(learning_objects_hash) || []
    end

    # Usage:
    #   learning_object = repository.for(:learning_objects).get_by_dspace_id 123
    #
    def get_by_dspace_id(id_dspace)
      result = connection.query "SELECT FROM LearningObject WHERE id_dspace=#{id_dspace}"
      build_learning_object result.first
    end

    # Usage:
    #   learning_object = repository.for(:learning_objects).get_by_dspace_id 123
    #
    def find(id)
      result = connection.query "SELECT FROM #{id}"
      build_learning_object result.first
    end

    def create(name, url)
      connection.command sprintf("INSERT INTO LearningObject (name,URL) VALUES ('%s','%s')", name, url)
    end

    # Usage:
    #   repository.for(:learning_objects).destroy learning_object
    #
    def destroy(learning_object)
      connection.command sprintf("DELETE VERTEX LearningObject where @rid = '%s'", learning_object.id)
    end

    ##
    # To create "index:learningobject_search" on OrientDB, use the following command:
    #   CREATE INDEX learningobject_search
    #   ON LearningObject (name, description)
    #   FULLTEXT ENGINE LUCENE
    #   METADATA {"analyzer":"org.apache.lucene.analysis.br.BrazilianAnalyzer"}
    def search(qry)
      learning_objects_hash = connection.query "
                                SELECT EXPAND(rid)
                                FROM index:learningobject_search
                                WHERE key LUCENE '#{qry}'
                              ", limit: -1
      learning_objects = build_learning_objects(learning_objects_hash) || []
    end

    private

    def count_likes(learning_object)
      get_in_edges_count "Likes", learning_object.id
    end

    def count_views(learning_object)
      get_in_edges_count "Views", learning_object.id
    end

    def build_learning_object(args={})
      lo = LearningObject.new(:id => args["@rid"],
                              :name => args["name"],
                              :description => args["description"],
                              :thumbnail => args["thumbnail"],
                              :created_at => args["created_at"],
                              :id_dspace => args["id_dspace"],
                              :type => args["type"],
                              :bitstreams => args["bitstreams"],
                              :metadata => args["metadata"],
                              :last_modified => args["last_modified"])

      unless args["in_Likes"].nil?
        lo.likes = args["in_Likes"].count
      else
        lo.likes = 0
      end

      unless args["in_Views"].nil?
        lo.views = args["in_Views"].count
      else
        lo.views = 0
      end

      lo
    end

    def build_learning_objects(hash=[])
      learning_objects = []
      hash.each do |h|
        learning_objects << build_learning_object(h)
      end
      learning_objects
    end

  end
end
